/* Foefel.java -- code for analyzing Burp overview table of websocket traffic
 *
 * The base assumption for the code in this class is that Burp uses an encapsulation of
 * all websocket traffic properties and that that encapsulation is implemented in one single class
 * which serves as the basis for the TableModel required by the GUI JTable compoment that shows
 * an overview of recorded websocket messages. In other words: find the JTable component, get its
 * corresponding TableModel object which probably contains a property that is a list of some kind:
 * a list of the assumed websocket message encapsulation objects.
 *
 * The assumption turned out to be correct, but analysis of the TableModel object is not so straightforward
 * and requires additional techniques based on Java reflection and even dumping the classfiles using
 * "javap". Fortunately, the TableModel implementations require a method with name "getRowCount()" and
 * analysis of said method on the obfuscated Java classes points right away to which property contains
 * the actual data in list form.
 *
 * Unfortunately, all of that means that the code in this file is probably highly dependent on the
 * exact version of the Burp jar that was used for creating this extension: version 1.6
 *
 * However, there is hope that the functionality of this extension will become part of the next
 * release(s?) of Burp, so that there will be no need anymore to analyse future releases to keep this
 * code working (a man can hope, can't he? ;-) )
 *
 * Copyright (c) 2014 Eddy Vanlerberghe.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of Eddy Vanlerberghe shall not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY EDDY VANLERBERGHE ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL EDDY VANLERBERGHE BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package evilwan;

import java.awt.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import javax.swing.JPanel;
import java.util.*;
import java.io.*;
import javax.swing.event.*;
import java.lang.reflect.*;

public class Foefel {
    // {{ Class members }}

    // {{ Instance members }}
    
    // {{ Class methods }}

    /**
     * Helper method for printing messages.
     * <p>
     * The printed message is written to <code>System.out</code> and starts with the current
     * class name, followed by two dashes, followed by the specified text.
     * @param s contains the <code>String</code> value to print.
     */
    private static void say(String s) {
	System.out.println("Foefel -- " + s);
    }
    //
    // Utility methods for dumping various types of static properties.
    //
    /**
     * Format a string using more or less standard Java syntax for special characters.
     */
    private static String s2s(String s) {
	StringBuilder sb = new StringBuilder();
	for(int i = 0; i < s.length(); ++i) {
	    char c = s.charAt(i);
	    if((c == '\\') || (c == '\'') || (c == '\"')){
		sb.append('\\');
		sb.append(c);
	    } else if(c == '\n') {
		sb.append('\\');
		sb.append('n');
	    } else if(c == '\r') {
		sb.append('\\');
		sb.append('r');
	    } else if(c == '\f') {
		sb.append('\\');
		sb.append('f');
	    } else if(c == '\t') {
		sb.append('\\');
		sb.append('t');
	    } else if(((int)c < 0x20) || ((int)c > 0x7E)) {
		sb.append(String.format("\\u%04x", (int)c));
	    } else {
		sb.append(c);
	    }
	}
	return sb.toString();
    }
    /**
     * Format a byte[] showing both hex values and character representations (if applicable)
     */
    private static String b2s(byte[] bar) {
	StringBuilder sb = new StringBuilder();
	StringBuilder sa = new StringBuilder();
	for(int i = 0; i < bar.length; ++i) {
	    if((i % 16) == 0) {
		if(i > 0) {
		    sb.append("\t// ");
		    sb.append(sa.toString());
		    sb.append('\n');
		    sa.delete(0, sa.length());
		}
		sb.append('\t');
	    }
	    int byt = ((int)bar[i]) & 0xff;
	    sb.append(String.format("0x%02x, ", byt));
	    if((byt >= 0x20) && (byt < 0x7f)) {
		sa.append((char) byt);
	    } else {
		sa.append('.');
	    }
	}
	if(sa.length() > 0) {
	    sb.append("\t// ");
	    sb.append(sa.toString());
	}
	return sb.toString();
    }
    /**
     * Format a char[] using more or less standard Java syntax for special characters.
     */
    private static String c2s(char[] s) {
	StringBuilder sb = new StringBuilder();
	for(int i = 0; i < s.length; ++i) {
	    char c = s[i];
	    if((c == '\\') || (c == '\'') || (c == '\"')){
		sb.append('\\');
		sb.append(c);
	    } else if(c == '\n') {
		sb.append('\\');
		sb.append('n');
	    } else if(c == '\r') {
		sb.append('\\');
		sb.append('r');
	    } else if(c == '\f') {
		sb.append('\\');
		sb.append('f');
	    } else if(c == '\t') {
		sb.append('\\');
		sb.append('t');
	    } else if(((int)c < 0x20) || ((int)c > 0x7E)) {
		sb.append(String.format("\\u%04x", (int)c));
	    } else {
		sb.append(c);
	    }
	}
	return sb.toString();
    }
    //
    // Various GUI component analysis methods
    //
    private static Field findListProperty(Object o) {
	Field fld = null;
	try {
	    Class c = o.getClass();
	    Field[] flds = c.getDeclaredFields();
	    for(int i = 0; i < flds.length; ++i) {
		flds[i].setAccessible(true);	// Make sure we can access the field
		if(flds[i].getType().equals(ArrayList.class)) {
		    if(fld != null) {
			throw new Exception("Instance of " + o.getClass().getName() +
					    " contains multiple List properties: found " +
					    fld.toGenericString() + " and " +
					    flds[i].toGenericString());
		    }
		    fld = flds[i];
		}
	    }
	} catch(Throwable t) {
	    System.out.println("## caught: " + t);
	    t.printStackTrace();
	    System.exit(1);
	}
	return fld;
    }

    private static JTable analyzeJTable(JTable tab) {
	TableModel model = tab.getModel();
	int n = model.getColumnCount();
	for(int i = 0; i < n; ++i) {
	    String colnam = model.getColumnName(i);
	    //
	    // Hack: in current release the only table with a column "Direction" is
	    // the JTable for websocket traffic
	    //
	    if(colnam.equals("Direction")) {
		//
		// Gotcha!
		//
		//_websockets_table = tab;
		return tab;
	    }
	}
	return analyzeKids(tab);
    }

    private static JTable analyzeKids(Container box) {
	Component[] parts = box.getComponents();
	JTable jt = null;
	for(int i = 0; i < parts.length; ++i) {
	    if(parts[i] instanceof JTable) {
		jt = analyzeJTable((JTable) parts[i]);
	    } else if(parts[i] instanceof Container) {
		jt = analyzeKids((Container) parts[i]);
	    }
	    if(jt != null) {
		return jt;
	    }
	}
	return jt;
    }

    public static Object findWSTable() throws Exception {
	JTable jt = null;
	//
	// Find top-level Burp GUI Frame
	//
	Frame[] frames = Frame.getFrames();
	for(int i = 0; i < frames.length; ++i) {
	    if(frames[i].getTitle().startsWith("Burp Suite")) {
		//
		// Got it: now analyze entire component tree until we
		// find the JTable component containing the websocket packet list
		//
		jt = analyzeKids(frames[i]);
		break;
	    }
	}
	//
	// Now find remaining pieces of information from Burp environment: we intend
	// to find some sort of List-type property in the TableModel linked to the JTable
	// object that we found in the GUI. This can be done by simply dumping the full class name
	// of the TableModel of the JTable object that we have in "_websockets_table" (for Burp 1.6
	// this results in "burp.dzb" but this is most likely different for each build of Burp)
	//
	// Using javap on "burp.dzb" shows only one List-type property (an ArrayList) and examining the
	// bytecode for method "burp.dzb.getRowCount()" confirms that the "size()" of that ArrayList property
	// is returned as rowcount in the TableModel. Again, in Burp 1.6, the name of that property is "t" (that
	// name is not required in the analysis code below, just that it's an ArrayList property)
	//
	// The assumption is that whatever is stored in that ArrayList, is some sort of encapsulation of all
	// data relevant for each websocket packet (timestamp, direction, original message data, modified message
	// data etc.) So, next we need to get hold of that ArrayList and then we need to find a way to
	// interpret the contents of each of its elements.
	//
	// Note that, thanks to reflection, we do have access to the private ArrayList property after
	// using the Field.setAccessible(true) call :-)
	//
	// Update for 1.6.01: "burp.dzb" became "burp.kwd"; the ArrayList fieldname remains "t".
	//
	TableModel model = jt.getModel();
	//say("TableModel model is a " + model.getClass().getName());
	Field f = findListProperty(model);
	//say("Field f is a " + f.toGenericString());
	Object ooo = f.get(model);
	//say("Object ooo is a " + ooo.getClass().getName());
	return ooo;	// return actual ArrayList property
    }
    /**
     * Return a list of static class properties that are either strings,
     * character arrays or byte arrays.
     */
    public static Vector<String> getStaticStrings(Class c) throws Exception {
	String clsnam = c.getName();
	int jj = clsnam.lastIndexOf(".");
	String shortname = clsnam.substring(jj + 1);
	Vector<String> v = new Vector<String>();
	Field[]	flds = c.getDeclaredFields();
	for(int i=0; i < flds.length; ++i) {
	    flds[i].setAccessible(true);	// Make sure we can access the field
	    String fldnam = flds[i].getName();
	    if("[Ljava.lang.String;".equals(flds[i].getType().getName())) {
		//
		// String array
		//
		try {
		    Object o = flds[i].get(null);
		    for(int j = 0; j < Array.getLength(o); ++j) {
			v.add(shortname + "." + fldnam + "[" + j + "]=\"" + s2s((String)Array.get(flds[i].get(null), j)) + "\";");
		    }
		} catch(NullPointerException npe) {
		    String msg = clsnam + "." + fldnam + " not available: modifiers = " + flds[i].getModifiers() +
			" (" + Modifier.toString(flds[i].getModifiers()) + ")";
		    say(msg);
		    v.add(msg);
		}
	    } else if ("java.lang.String".equals(flds[i].getType().getName())) {
		//
		// Scalar String
		//
		try {
		    v.add(shortname + "." + fldnam + "=\"" + s2s((String)flds[i].get(null)) + "\";");
		} catch(NullPointerException npe) {
		    String msg = clsnam + "." + fldnam + " not available: modifiers = " + flds[i].getModifiers() +
			" (" + Modifier.toString(flds[i].getModifiers()) + ")";
		    say(msg);
		    v.add(msg);
		}
	    } else if ("[B".equals(flds[i].getType().getName())) {
		//
		// byte array
		//
		try {
		    v.add(shortname + "." + fldnam + "={\n" + b2s((byte[])flds[i].get(null)) + "\n};");
		} catch(NullPointerException npe) {
		    String msg = clsnam + "." + fldnam + " not available: modifiers = " + flds[i].getModifiers() +
			" (" + Modifier.toString(flds[i].getModifiers()) + ")";
		    say(msg);
		    v.add(msg);
		}
	    } else if ("[C".equals(flds[i].getType().getName())) {
		//
		// char array
		//
		try {
		    v.add(shortname + "." + fldnam + "={\n" + c2s((char[])flds[i].get(null)) + "\n};");
		} catch(NullPointerException npe) {
		    String msg = clsnam + "." + fldnam + " not available: modifiers = " + flds[i].getModifiers() +
			" (" + Modifier.toString(flds[i].getModifiers()) + ")";
		    say(msg);
		    v.add(msg);
		}
	    }
	}
	return v;
    }

    public static void dumpStatics(Class c) {
	try {
	    Vector<String> strings = getStaticStrings(c);
	    for(Enumeration<String> e = strings.elements(); e.hasMoreElements(); ) {
		say(e.nextElement());
	    }
	} catch (Exception ex) {
	    say("caught: " + ex);
	    ex.printStackTrace();
	}
    }

    // {{ Constructors }}
    
    
    // {{ Instance methods }}

}
