/* Goewie.java - Helper code for analyzing the Burp intercept GUI
 *
 * The methods in this class are mostly static. They are based on analysis of a dump
 * of the entire Burp GUI component tree.
 *
 * Copyright (c) 2014 Eddy Vanlerberghe.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of Eddy Vanlerberghe shall not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY EDDY VANLERBERGHE ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL EDDY VANLERBERGHE BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

package evilwan;

import burp.*;

import java.awt.Frame;
import java.awt.Container;
import java.awt.Component;
import javax.swing.JLabel;
import javax.swing.AbstractButton;
import javax.swing.JButton;
import javax.swing.JTextField;
import javax.swing.JToggleButton;
import javax.swing.JTextArea;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.table.TableModel;
import javax.swing.event.TableModelListener;
import javax.swing.event.TableModelEvent;
import javax.swing.JPanel;
import java.awt.LayoutManager;
import java.util.EventListener;
import java.util.Stack;
import java.util.ArrayList;
import javax.swing.event.ListSelectionListener;
import javax.swing.DefaultListSelectionModel;
import java.awt.Rectangle;

public class Goewie {
    // {{ Class members }}

    // {{ Instance members }}

    // {{ Class methods }}

    /**
     * Find the main application frame (use its title for locating which Frame)
     */
    public static Frame findTopFrame() throws Exception {
	Frame[] frames = Frame.getFrames();
	for(int i = 0; i < frames.length; ++i) {
	    if(frames[i].getTitle().startsWith("Burp Suite")) {
		return frames[i];
	    }
	}
	return null;
    }
    /**
     * Find the <code>JTabbedPane</code> containing a tab with the specified name.
     * <p>
     * Main reason for using this method is to be able to select the named tab later on.
     */
    public static JTabbedPane findTabbedPane(Container c, String tabnam) throws Exception {
	if(c instanceof JTabbedPane) {
	    JTabbedPane tp = (JTabbedPane) c;
	    int n = tp.getTabCount();
	    for(int i = 0; i < n; ++i) {
		if(tabnam.equals(tp.getTitleAt(i))) {
		    tp.setSelectedIndex(i);
		    return tp;
		}
	    }
	}
	Component[] parts = c.getComponents();
	for(int i = 0; i < parts.length; ++i) {
	    if(parts[i] instanceof Container) {
		JTabbedPane o = findTabbedPane((Container)parts[i], tabnam);
		if(o != null) {
		    return o;
		}
	    }
	}
	return null;
    }
    /**
     * Find component in <code>JTabbedPane</code> that belongs to the specified tab.
     */
    public static Component findNamedTabPane(Container c, String tabnam) throws Exception {
	if(c instanceof JTabbedPane) {
	    JTabbedPane tp = (JTabbedPane) c;
	    int n = tp.getTabCount();
	    for(int i = 0; i < n; ++i) {
		if(tabnam.equals(tp.getTitleAt(i))) {
		    return tp.getComponentAt(i);
		}
	    }
	}
	Component[] parts = c.getComponents();
	for(int i = 0; i < parts.length; ++i) {
	    if(parts[i] instanceof Container) {
		Component o = findNamedTabPane((Container)parts[i], tabnam);
		if(o != null) {
		    return o;
		}
	    }
	}
	return null;
    }
    /**
     * Select named tab in <code>JTabbedPane</code>
     */
    public static int selectNamedTabPane(Container c, String tabnam) throws Exception {
	if(c instanceof JTabbedPane) {
	    JTabbedPane tp = (JTabbedPane) c;
	    int n = tp.getTabCount();
	    for(int i = 0; i < n; ++i) {
		if(tabnam.equals(tp.getTitleAt(i))) {
		    tp.setSelectedIndex(i);
		    return i;
		}
	    }
	}
	Component[] parts = c.getComponents();
	for(int i = 0; i < parts.length; ++i) {
	    if(parts[i] instanceof Container) {
		int o = selectNamedTabPane((Container)parts[i], tabnam);
		if(o < 0) {
		    return -1;
		}
	    }
	}
	return -1;
    }
    /**
     * Find the <code>JLabel</code> component containing the descriptive text for the intercepted message.
     * <p>
     * This code is based on analyzing the GUI components at run-time.
     */
    public static JLabel findMessageLabel(JPanel c) throws Exception {
	JPanel pnlwrk = null;
	Component[] parts = c.getComponents();
	for(int i = 0; i < parts.length; ++i) {
	    if((parts[i] instanceof JPanel) && ((int)parts[i].getBounds().getX() == 0) &&
	       ((int)parts[i].getBounds().getY() == 0)){
		pnlwrk = (JPanel) parts[i];
		break;
	    }
	}
	if(pnlwrk == null) {
	    throw new Exception("Failed to find JPanel at (0,0) in tabbed pane tab");
	}
	JPanel pnl = null;
	parts = pnlwrk.getComponents();
	for(int i = 0; i < parts.length; ++i) {
	    if((parts[i] instanceof JPanel) && ((int)parts[i].getBounds().getX() == 0) &&
	       ((int)parts[i].getBounds().getY() == 0)){
		pnl = (JPanel) parts[i];
		break;
	    }
	}
	if(pnl == null) {
	    throw new Exception("Failed to find JPanel at (0,0) surrounding message JLabel");
	}
	parts = pnl.getComponents();
	for(int i = 0; i < parts.length; ++i) {
	    if(parts[i] instanceof JLabel) {
		 return (JLabel) parts[i];
	    }
	}
	return null;
    }

    /**
     * Look for <code>JButton</code> with a label that contains the specified name.
     */
    public static AbstractButton findNamedButton(Component c, String btnnam) throws Exception {
	if(c instanceof AbstractButton) {
	    AbstractButton btn = (AbstractButton) c;
	    if(btn.getText().indexOf(btnnam) >= 0) {
		return btn;
	    }
	}
	if(c instanceof Container) {
	    Component[] parts = ((Container)c).getComponents();
	    for(int i = 0; i < parts.length; ++i) {
		AbstractButton o = (AbstractButton) findNamedButton(parts[i], btnnam);
		if(o != null) {
		    return o;
		}
	    }
	}
	return null;
    }
    /**
     * Find first <code>JTable</code> in container
     */
    public static JTable findJTable(Container c) throws Exception {
	if(c instanceof JTable) {
	    return (JTable)c;
	}
	Component[] parts = c.getComponents();
	for(int i = 0; i < parts.length; ++i) {
	    if(parts[i] instanceof Container) {
		JTable o = findJTable((Container)parts[i]);
		if(o != null) {
		    return o;
		}
	    }
	}
	return null;
    }


    // {{ Constructors }}
    
    
    // {{ Instance methods }}
}
