#!/usr/bin/env perl
#
# Quick'nd dirty script to find out the names for obfuscated method names
# in the Burp class that represents a WebSocket message.
#
# Use this script in combination with running Burp with the "-XXX" commandline
# parameter and the WS extension loaded. While Burp is running, visit a website that uses
# WebSocket messages and then select to save them in a file: that action will trigger
# debug code in the extension to print the data that is needed by this filter script.
#
# In order to automate the process, a Java decompiler is used rather than doing manual
# interpretation of javap output (bytecode level): Procyon (https://bitbucket.org/mstrobel/procyon)
#
# The input should look something like this:
#
#    WSPacket -- init() -- _wsclazz is: burp.owd
#    Foefel -- burp.owd.f not available: modifiers = 2 (private)
#    Foefel -- burp.owd.f not available: modifiers = 2 (private)
#    Foefel -- owd.z[0]="editedMessage";
#    Foefel -- owd.z[1]="comment";
#    Foefel -- owd.z[2]="originalMessage";
#    Foefel -- owd.z[3]="clientToServer";
#    ...
#    Deleting temporary files - please wait ... done.
#
# The output should end with something like this:
#
#    ...
#    Method returning original message: c
#    Method returning edited message: f
#    Method returning comment: e
#
use strict;
#
# Name of new Burp jar to analyze: copy to same directory as this script.
#
my($BURP) = 'burpsuite_pro_v1.6.04.jar';
#
# Where to find the Procyon jar: copy to same directory as this script.
#
my($PROCY) = 'procyon-decompiler-0.5.25.jar';
#
# No more changes required below
#
sub say {
    print join(' ', @_), "\n";
}

my($fulcls) = '';
my($javacls) = '';
my($cls) = '';
my($tmpdir) = 'tmp_' . time();
my(%vars);

my($l);
while($l = <STDIN>) {
    chomp($l);
    if($l =~ /clazz is:\s*([^\s]+)/) {
	$fulcls = $1;
	$cls = $fulcls;
	$cls =~ s/^.*\.([^\.]+)$/\1/;
	$fulcls =~ s/\./\//g;
	$javacls = $fulcls;
	$fulcls .= '.class';
	$javacls .= '.procy';
    } elsif($l =~ /\s+([^\s]+)=(.*);\s*$/) {
	$vars{$1} = $2;
    }
}
#
# Create temporary directory for extraction of obfuscated class file
#
if(! -e $tmpdir) {
    mkdir($tmpdir);
}
#
# Extract obfuscated Burp class that represents a WebSocket message
#
my($cmd) = sprintf("(cd %s ; jar xvf ../%s %s)", $tmpdir, $BURP, $fulcls);
#&say('Extract from jar command:', $cmd);
system($cmd);
#
# Decompile the extracted class and replace all obfuscated strings with the
# the actual values at runtime that were dumped by the -XXX option
#
$cmd = sprintf("(cd %s ; java -jar ../%s -eta  %s ", $tmpdir, $PROCY, $fulcls);
my($ky);
foreach $ky (keys(%vars)) {
    my($v) = $vars{$ky};
    $ky =~ s/([\[\]])/\\\1/g;
    $v =~ s/\"/\\\"/g;
    $cmd .= sprintf("| sed \'s/%s/%s/g\'", $ky, $v);
}
$cmd .= ' > '.$javacls.')';
#&say('Decompile command:', $cmd);
system($cmd);
#
# Slurp decompiled Java file
#
my($java) = '';
open(F, '<'.$tmpdir.'/'.$javacls) || die "failed to open decompiled java file: $!\n";
while($l = <F>) {
    $java .= $l;
}
close(F);
#
# Look for lines that are of the format "keyword" ... this.X" where "X"
# starts with an alphabetic letter and is followed by zero or more alphanumeric
# characters.
#
# These lines are used to find out the names of the corresponding obfuscated
# property names.
#
$java =~ /\"originalMessage\".*this\.([a-z][a-z0-9]*)/i;
my($orgmsg) = $1;
my($reorgmsg) = qr/$1/;

$java =~ /\"editedMessage\".*this\.([a-z][a-z0-9]*)/i;
my($edtmsg) = $1;
my($reedtmsg) = qr/$1/;

$java =~ /\"comment\".*this\.([a-z][a-z0-9]*)/i;
my($comment) = $1;
my($recomment) = qr/$1/;
#
# Hack: find out the internal type of original/edited message data
#
$java =~ /private\s+(final)?\s*([^\s]+)\s+${$reorgmsg}\s*;/si;
my($wstyp) = $2;
my($rewstyp) = qr/$2/;
#
# Finally search for accessor methods that return the properties
# we are looking for. Use those name in Java class evilwan/WSPacket.
#
$java =~ /public\s+${rewstyp}\s+([a-z])\(\)\s+{\s+return\s+this\.${reorgmsg}\s*;/si;
&say('Method returning original message:', $1);

$java =~ /public\s+${rewstyp}\s+([a-z])\(\)\s+{\s+return\s+this\.${reedtmsg}\s*;/si;
&say('Method returning edited message:', $1);

$java =~ /public\s+String\s+([a-z])\(\)\s+{\s+return\s+this\.${comment}\s*;/si;
&say('Method returning comment:', $1);
